--- apps/app_voicemail.c	2004-09-02 00:08:40.000000000 -0700
+++ apps/app_voicemail.c.optional-manditory-password-change	2004-09-02 00:11:39.000000000 -0700
@@ -258,6 +258,7 @@
 static int maxgreet;
 static int skipms;
 static int maxlogins;
+static char forcepwchange[80];	/* Force change if it's set to this */
 
 static int reviewvm;
 static int calloper;
@@ -3495,6 +3496,9 @@
 	char *context=NULL;
 	int silentexit = 0;
 
+	char newpassword[80] = "";
+	char newpassword2[80] = "";
+                        
 	LOCAL_USER_ADD(u);
 	memset(&vms, 0, sizeof(vms));
 	strncpy(fmtc, vmfmts, sizeof(fmtc) - 1);
@@ -3613,6 +3617,59 @@
 		mkdir(vms.curdir, 0700);
 		snprintf(vms.curdir, sizeof(vms.curdir), "%s/voicemail/%s/%s", (char *)ast_config_AST_SPOOL_DIR, vmu->context, vms.username);
 		mkdir(vms.curdir, 0700);
+
+		/* Check if we should force the user to immediately change their password */
+		if (!strcmp(vmu->password, forcepwchange)) {
+			if (option_verbose > 2)
+				ast_verbose( VERBOSE_PREFIX_3 "Forcing immediate password change for user '%s'\n", vms.username);
+			res = ast_streamfile(chan, "vm-mustchangepasswd",chan->language);
+			res = ast_waitstream(chan, "");
+			if (res > 0)
+				res = 0;
+
+			/* XXX this is borrowed wholesale from vm_options - it should probably be removed from both and placed in it's own function XXX */
+			newpassword[1] = '\0';
+			newpassword[0] = cmd = play_and_wait(chan,"vm-newpassword");
+			if (cmd < 0) {
+				valid = 0;
+			}
+			if ((cmd = ast_readstring(chan,newpassword + strlen(newpassword),sizeof(newpassword)-1,2000,10000,"#")) < 0) {
+				valid = 0;
+			}
+			if (valid) {
+				newpassword2[1] = '\0';
+				newpassword2[0] = cmd = play_and_wait(chan,"vm-reenterpassword");
+				if (cmd < 0) {
+					valid = 0;
+				} 
+				if ((cmd = ast_readstring(chan,newpassword2 + strlen(newpassword2),sizeof(newpassword2)-1,2000,10000,"#"))) {
+					valid = 0;
+				}
+				if (strcmp(newpassword, newpassword2)) {
+					ast_log(LOG_NOTICE,"Password mismatch for user %s (%s != %s)\n", vms.username, newpassword, newpassword2);
+					res = ast_streamfile(chan, "vm-mismatch", chan->language);
+					valid = 0;
+				}
+			}
+			if (valid) {
+				vm_change_password(vmu,newpassword);
+				ast_log(LOG_DEBUG,"User %s set password to %s of length %i\n", vms.username, newpassword, (int)strlen(newpassword));
+				res = ast_streamfile(chan,"vm-passchanged", chan->language);
+				if (!res)
+					res = ast_waitstream(chan, "");
+			}
+			if (!valid) {
+				res = ast_waitstream(chan, "");
+				if (res > 0)
+					res = 0;
+				ast_stopstream(chan);
+				res = ast_streamfile(chan, "vm-goodbye", chan->language);
+				if (res > 0)
+					res = 0;
+				goto out;
+			}                                                
+		}
+		
 		/* Retrieve old and new message counts */
 		open_mailbox(&vms, vmu, 1);
 		vms.oldmessages = vms.lastmsg + 1;
@@ -4293,6 +4350,13 @@
 				ast_log(LOG_WARNING, "Invalid max failed login attempts\n");
 			}
 		}
+ 
+		if ((s = ast_variable_retrieve(cfg, "general", "forcepwchange"))) {
+			strncpy(forcepwchange, s, sizeof(forcepwchange) - 1);
+			ast_log(LOG_DEBUG, "will require password change if it matches: %s\n", forcepwchange);
+		} else {
+			forcepwchange[0] = '\0';
+		}
 
 		if ((s = ast_variable_retrieve(cfg, "general", "cidinternalcontexts"))){
 			ast_log(LOG_DEBUG,"VM_CID Internal context string: %s\n",s);
